@@ -15,11 +15,12 @@ app.command()(mod.main)
 
 def test_not_exists(tmpdir):
     config_file = Path(tmpdir) / "config.txt"
-    if config_file.exists():  # pragma no cover
+    if config_file.exists():  # pragma: no cover
         config_file.unlink()
     result = runner.invoke(app, ["--config", f"{config_file}"])
     assert result.exit_code != 0
-    assert "Invalid value for '--config': File" in result.output
+    assert "Invalid value for '--config'" in result.output
+    assert "File" in result.output
     assert "does not exist" in result.output
 
 
@@ -35,14 +36,14 @@ def test_exists(tmpdir):
 def test_dir():
     result = runner.invoke(app, ["--config", "./"])
     assert result.exit_code != 0
-    assert "Invalid value for '--config': File './' is a directory." in result.output
+    assert "Invalid value for '--config'" in result.output
+    assert "File './' is a directory." in result.output
 
 
 def test_script():
     result = subprocess.run(
         [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
+        capture_output=True,
         encoding="utf-8",
     )
     assert "Usage" in result.stdout
